<?php

namespace Drupal\routing_on_exception\EventSubscriber;

use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\Routing\RouteBuildEvent;
use Drupal\Core\Routing\RouteMatch;
use Drupal\Core\Routing\RoutingEvents;
use Drupal\Core\Url;
use Drupal\routing_on_exception\RoutingOnExceptionManagerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpKernel\Event\ExceptionEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Redirects users when access is denied due to CSRF Token validation failure.
 */
class RoutingOnExceptionSubscriber implements EventSubscriberInterface
{

  private RoutingOnExceptionManagerInterface $routeExceptionRedirectsManager;

  private LoggerChannelInterface $logger;

  function __construct(
    RoutingOnExceptionManagerInterface $routeExceptionRedirectsManager,
    LoggerChannelInterface $logger
  ) {
    $this->routeExceptionRedirectsManager = $routeExceptionRedirectsManager;
    $this->logger = $logger;
  }

  public function onRouterAlterValidateOnExceptionOption(RouteBuildEvent $event)
  {
    $routes = $event->getRouteCollection();
    $this->routeExceptionRedirectsManager->validateOnExceptionsOption($routes);
  }

  public function onExceptionApplyRoutingOnExceptionActions(ExceptionEvent $event)
  {
    $rule_actions = $this->routeExceptionRedirectsManager->getRouteExceptionActions(
      RouteMatch::createFromRequest($event->getRequest()),
      $event->getThrowable()
    );
    foreach ($rule_actions as $actions) {
      foreach ($actions as $action => $value) {
        switch ($action) {
          case 'log':
            $this->logger->error($value);
            break;
          case 'redirect':
            $event->setResponse(new RedirectResponse(Url::fromRoute($value)->toString()));
            break;
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents()
  {
    // Use a lower priority than
    // \Drupal\Core\EventSubscriber\ExceptionLoggingSubscriber, so Drupal still logs
    // access denied errors.
    $events[KernelEvents::EXCEPTION][] = ['onExceptionApplyRoutingOnExceptionActions', 40];
    $events[RoutingEvents::ALTER][] = ['onRouterAlterValidateOnExceptionOption', 0];
    return $events;
  }

}
